@@ -236,22 +236,24 @@ module Hledger.Cli.Balance (
   balancemode
  ,balance
  ,balanceReportAsText
+ ,balanceReportItemAsText
  ,periodBalanceReportAsText
  ,cumulativeBalanceReportAsText
  ,historicalBalanceReportAsText
  ,tests_Hledger_Cli_Balance
 ) where
 
+import Data.List (intercalate)
+import Data.Maybe (fromMaybe, isJust)
 import System.Console.CmdArgs.Explicit as C
 import Text.CSV
 import Test.HUnit
 import Text.Printf (printf)
 import Text.Tabular as T
-import Text.Tabular.AsciiArt
+import Text.Tabular.AsciiWide
 
 import Hledger
-import Hledger.Data.OutputFormat
-import Hledger.Cli.Options
+import Hledger.Cli.CliOptions
 import Hledger.Cli.Utils
 
 
@@ -263,13 +265,14 @@ balancemode = (defCommandMode $ ["balance"] ++ aliases) { -- also accept but don
       flagNone ["tree"] (\opts -> setboolopt "tree" opts) "show accounts as a tree (default in simple reports)"
      ,flagNone ["flat"] (\opts -> setboolopt "flat" opts) "show accounts as a list (default in multicolumn mode)"
      ,flagReq  ["drop"] (\s opts -> Right $ setopt "drop" s opts) "N" "flat mode: omit N leading account name parts"
-     ,flagReq  ["format"] (\s opts -> Right $ setopt "format" s opts) "FORMATSTR" "tree mode: use this custom line format"
+     ,flagReq  ["format"] (\s opts -> Right $ setopt "format" s opts) "FORMATSTR" "singlecolumn mode: use this custom line format"
      ,flagNone ["no-elide"] (\opts -> setboolopt "no-elide" opts) "tree mode: don't squash boring parent accounts"
      ,flagNone ["historical","H"] (\opts -> setboolopt "historical" opts) "multicolumn mode: show historical ending balances"
      ,flagNone ["cumulative"] (\opts -> setboolopt "cumulative" opts) "multicolumn mode: show accumulated ending balances"
      ,flagNone ["average","A"] (\opts -> setboolopt "average" opts) "multicolumn mode: show a row average column"
      ,flagNone ["row-total","T"] (\opts -> setboolopt "row-total" opts) "multicolumn mode: show a row total column"
      ,flagNone ["no-total","N"] (\opts -> setboolopt "no-total" opts) "don't show the final total row"
+     ,flagNone ["value","V"] (setboolopt "value") "show amounts as their current market value in their default valuation commodity"
      ]
      ++ outputflags
     ,groupHidden = []
@@ -288,25 +291,38 @@ balance opts@CliOpts{reportopts_=ropts} j = do
       let format   = outputFormatFromOpts opts
           interval = intervalFromOpts ropts
           baltype  = balancetype_ ropts
+          valuedate = fromMaybe d $ queryEndDate False $ queryFromOpts d ropts
       case interval of
         NoInterval -> do
           let report = balanceReport ropts (queryFromOpts d ropts) j
+              convert | value_ ropts = balanceReportValue j valuedate
+                      | otherwise    = id
               render = case format of
                 "csv" -> \ropts r -> (++ "\n") $ printCSV $ balanceReportAsCsv ropts r
                 _     -> balanceReportAsText
-          writeOutput opts $ render ropts report
+          writeOutput opts $ render ropts $ convert report
         _ -> do
           let report = multiBalanceReport ropts (queryFromOpts d ropts) j
+              convert | value_ ropts = multiBalanceReportValue j valuedate
+                      | otherwise    = id
               render = case format of
                 "csv" -> \ropts r -> (++ "\n") $ printCSV $ multiBalanceReportAsCsv ropts r
                 _     -> case baltype of
                   PeriodBalance     -> periodBalanceReportAsText
                   CumulativeBalance -> cumulativeBalanceReportAsText
                   HistoricalBalance -> historicalBalanceReportAsText
-          writeOutput opts $ render ropts report
+          writeOutput opts $ render ropts $ convert report
 
 -- single-column balance reports
 
+-- | Find the best commodity to convert to when asked to show the
+-- market value of this commodity on the given date. That is, the one
+-- in which it has most recently been market-priced, ie the commodity
+-- mentioned in the most recent applicable historical price directive
+-- before this date.
+-- defaultValuationCommodity :: Journal -> Day -> Commodity -> Maybe Commodity
+-- defaultValuationCommodity j d c = mpamount <$> commodityValue j d c
+
 -- | Render a single-column balance report as CSV.
 balanceReportAsCsv :: ReportOpts -> BalanceReport -> CSV
 balanceReportAsCsv opts (items, total) =
@@ -321,15 +337,26 @@ balanceReportAsCsv opts (items, total) =
 balanceReportAsText :: ReportOpts -> BalanceReport -> String
 balanceReportAsText opts ((items, total)) = unlines $ concat lines ++ t
   where
-      lines = case lineFormatFromOpts opts of
-                Right f -> map (balanceReportItemAsText opts f) items
-                Left err -> [[err]]
+      fmt = lineFormatFromOpts opts
+      lines = case fmt of
+                Right fmt -> map (balanceReportItemAsText opts fmt) items
+                Left err  -> [[err]]
       t = if no_total_ opts
            then []
-           else ["--------------------"
-                 -- TODO: This must use the format somehow
-                ,padleft 20 $ showMixedAmountWithoutPrice total
-                ]
+           else
+             case fmt of
+               Right fmt ->
+                let
+                  -- abuse renderBalanceReportItem to render the total with similar format
+                  acctcolwidth = maximum' [length fullname | ((fullname, _, _), _) <- items]
+                  totallines = map rstrip $ renderBalanceReportItem fmt (replicate (acctcolwidth+1) ' ', 0, total)
+                  -- with a custom format, extend the line to the full report width;
+                  -- otherwise show the usual 20-char line for compatibility
+                  overlinewidth | isJust (format_ opts) = maximum' $ map length $ concat lines
+                                | otherwise             = defaultTotalFieldWidth
+                  overline   = replicate overlinewidth '-'
+                in overline : totallines
+               Left _ -> []
 
 tests_balanceReportAsText = [
   "balanceReportAsText" ~: do
@@ -356,40 +383,58 @@ This implementation turned out to be a bit convoluted but implements the followi
               EUR -1
     b         USD -1  ; Account 'b' has two amounts. The account name is printed on the last line.
 -}
--- | Render one balance report line item as plain text suitable for console output.
-balanceReportItemAsText :: ReportOpts -> [OutputFormat] -> BalanceReportItem -> [String]
-balanceReportItemAsText opts format ((_, accountName, depth), Mixed amounts) =
-    -- 'amounts' could contain several quantities of the same commodity with different price.
-    -- In order to combine them into single value (which is expected) we take the first price and
-    -- use it for the whole mixed amount. This could be suboptimal. XXX
-    let Mixed normAmounts = normaliseMixedAmountSquashPricesForDisplay (Mixed amounts) in
-    case normAmounts of
-      [] -> []
-      [a] -> [formatBalanceReportItem opts (Just accountName) depth a format]
-      (as) -> multiline as
-    where
-      multiline :: [Amount] -> [String]
-      multiline []     = []
-      multiline [a]    = [formatBalanceReportItem opts (Just accountName) depth a format]
-      multiline (a:as) = (formatBalanceReportItem opts Nothing depth a format) : multiline as
+-- | Render one balance report line item as plain text suitable for console output (or
+-- whatever string format is specified). Note, prices will not be rendered, and
+-- differently-priced quantities of the same commodity will appear merged.
+-- The output will be one or more lines depending on the format and number of commodities.
+balanceReportItemAsText :: ReportOpts -> StringFormat -> BalanceReportItem -> [String]
+balanceReportItemAsText opts fmt ((_, accountName, depth), amt) =
+  renderBalanceReportItem fmt (
+    maybeAccountNameDrop opts accountName,
+    depth,
+    normaliseMixedAmountSquashPricesForDisplay amt
+    )
 
-formatBalanceReportItem :: ReportOpts -> Maybe AccountName -> Int -> Amount -> [OutputFormat] -> String
-formatBalanceReportItem _ _ _ _ [] = ""
-formatBalanceReportItem opts accountName depth amount (fmt:fmts) =
-  s ++ (formatBalanceReportItem opts accountName depth amount fmts)
+-- | Render a balance report item using the given StringFormat, generating one or more lines of text.
+renderBalanceReportItem :: StringFormat -> (AccountName, Int, MixedAmount) -> [String]
+renderBalanceReportItem fmt (acctname, depth, total) =
+  lines $
+  case fmt of
+    OneLine comps       -> concatOneLine      $ render1 comps
+    TopAligned comps    -> concatBottomPadded $ render comps
+    BottomAligned comps -> concatTopPadded    $ render comps
   where
-    s = case fmt of
-         FormatLiteral l -> l
-         FormatField ljust min max field  -> formatField opts accountName depth amount ljust min max field
+    render1 = map (renderComponent1 (acctname, depth, total))
+    render  = map (renderComponent (acctname, depth, total))
 
-formatField :: ReportOpts -> Maybe AccountName -> Int -> Amount -> Bool -> Maybe Int -> Maybe Int -> HledgerFormatField -> String
-formatField opts accountName depth total ljust min max field = case field of
-        AccountField     -> formatValue ljust min max $ maybe "" (maybeAccountNameDrop opts) accountName
-        DepthSpacerField -> case min of
-                               Just m  -> formatValue ljust Nothing max $ replicate (depth * m) ' '
-                               Nothing -> formatValue ljust Nothing max $ replicate depth ' '
-        TotalField       -> formatValue ljust min max $ showAmountWithoutPrice total
-        _                  -> ""
+defaultTotalFieldWidth = 20
+
+-- | Render one StringFormat component for a balance report item.
+renderComponent :: (AccountName, Int, MixedAmount) -> StringFormatComponent -> String
+renderComponent _ (FormatLiteral s) = s
+renderComponent (acctname, depth, total) (FormatField ljust min max field) = case field of
+  DepthSpacerField -> formatString ljust Nothing max $ replicate d ' '
+                      where d = case min of
+                                 Just m  -> depth * m
+                                 Nothing -> depth
+  AccountField     -> formatString ljust min max acctname
+  TotalField       -> fitStringMulti min max True False $ showMixedAmountWithoutPrice total
+  _                -> ""
+
+-- | Render one StringFormat component for a balance report item.
+-- This variant is for use with OneLine string formats; it squashes
+-- any multi-line rendered values onto one line, comma-and-space separated,
+-- while still complying with the width spec.
+renderComponent1 :: (AccountName, Int, MixedAmount) -> StringFormatComponent -> String
+renderComponent1 _ (FormatLiteral s) = s
+renderComponent1 (acctname, depth, total) (FormatField ljust min max field) = case field of
+  AccountField     -> formatString ljust min max ((intercalate ", " . lines) (indented acctname))
+                      where
+                        -- better to indent the account name here rather than use a DepthField component
+                        -- so that it complies with width spec. Uses a fixed indent step size.
+                        indented = ((replicate (depth*2) ' ')++)
+  TotalField       -> fitStringMulti min max True False $ ((intercalate ", " . map strip . lines) (showMixedAmountWithoutPrice total))
+  _                -> ""
 
 -- multi-column balance reports
 
@@ -425,7 +470,7 @@ periodBalanceReportAsText opts r@(MultiBalanceReport (colspans, items, (coltotal
    render id (" "++) showMixedAmountOneLineWithoutPrice $
     addtotalrow $
      Table
-     (T.Group NoLine $ map (Header . padright acctswidth) accts)
+     (T.Group NoLine $ map (Header . padRightWide acctswidth) accts)
      (T.Group NoLine $ map Header colheadings)
      (map rowvals items')
   where
@@ -439,7 +484,7 @@ periodBalanceReportAsText opts r@(MultiBalanceReport (colspans, items, (coltotal
     renderacct ((a,a',i),_,_,_)
       | tree_ opts = replicate ((i-1)*2) ' ' ++ a'
       | otherwise  = maybeAccountNameDrop opts a
-    acctswidth = maximum $ map length $ accts
+    acctswidth = maximum' $ map strWidth accts
     rowvals (_,as,rowtot,rowavg) = as
                                    ++ (if row_total_ opts then [rowtot] else [])
                                    ++ (if average_ opts then [rowavg] else [])
@@ -459,7 +504,7 @@ cumulativeBalanceReportAsText opts r@(MultiBalanceReport (colspans, items, (colt
    render id (" "++) showMixedAmountOneLineWithoutPrice $
     addtotalrow $
      Table
-       (T.Group NoLine $ map (Header . padright acctswidth) accts)
+       (T.Group NoLine $ map (Header . padRightWide acctswidth) accts)
        (T.Group NoLine $ map Header colheadings)
        (map rowvals items)
   where
@@ -471,7 +516,7 @@ cumulativeBalanceReportAsText opts r@(MultiBalanceReport (colspans, items, (colt
     renderacct ((a,a',i),_,_,_)
       | tree_ opts = replicate ((i-1)*2) ' ' ++ a'
       | otherwise  = maybeAccountNameDrop opts a
-    acctswidth = maximum $ map length $ accts
+    acctswidth = maximum' $ map strWidth accts
     rowvals (_,as,rowtot,rowavg) = as
                                    ++ (if row_total_ opts then [rowtot] else [])
                                    ++ (if average_ opts then [rowavg] else [])
@@ -491,7 +536,7 @@ historicalBalanceReportAsText opts r@(MultiBalanceReport (colspans, items, (colt
    render id (" "++) showMixedAmountOneLineWithoutPrice $
     addtotalrow $
      Table
-       (T.Group NoLine $ map (Header . padright acctswidth) accts)
+       (T.Group NoLine $ map (Header . padRightWide acctswidth) accts)
        (T.Group NoLine $ map Header colheadings)
        (map rowvals items)
   where
@@ -503,7 +548,7 @@ historicalBalanceReportAsText opts r@(MultiBalanceReport (colspans, items, (colt
     renderacct ((a,a',i),_,_,_)
       | tree_ opts = replicate ((i-1)*2) ' ' ++ a'
       | otherwise  = maybeAccountNameDrop opts a
-    acctswidth = maximum $ map length $ accts
+    acctswidth = maximum' $ map strWidth accts
     rowvals (_,as,rowtot,rowavg) = as
                              ++ (if row_total_ opts then [rowtot] else [])
                              ++ (if average_ opts then [rowavg] else [])
