@@ -9,6 +9,7 @@ module Hledger.Cli.Register (
   registermode
  ,register
  ,postingsReportAsText
+ ,postingsReportItemAsText
  -- ,showPostingWithBalanceForVty
  ,tests_Hledger_Cli_Register
 ) where
@@ -18,10 +19,9 @@ import Data.Maybe
 import System.Console.CmdArgs.Explicit
 import Text.CSV
 import Test.HUnit
-import Text.Printf
 
 import Hledger
-import Hledger.Cli.Options
+import Hledger.Cli.CliOptions
 import Hledger.Cli.Utils
 
 
@@ -81,7 +81,12 @@ postingsReportItemAsCsvRecord (_, _, _, p, b) = [date,desc,acct,amt,bal]
 
 -- | Render a register report as plain text suitable for console output.
 postingsReportAsText :: CliOpts -> PostingsReport -> String
-postingsReportAsText opts = unlines . map (postingsReportItemAsText opts) . snd
+postingsReportAsText opts (_,items) = unlines $ map (postingsReportItemAsText opts amtwidth balwidth) items
+  where
+    amtwidth = maximum $ 12 : map (strWidth . showMixedAmount . itemamt) items
+    balwidth = maximum $ 12 : map (strWidth . showMixedAmount . itembal) items
+    itemamt (_,_,_,Posting{pamount=a},_) = a
+    itembal (_,_,_,_,a) = a
 
 tests_postingsReportAsText = [
   "postingsReportAsText" ~: do
@@ -112,24 +117,52 @@ tests_postingsReportAsText = [
 --
 -- date and description are shown for the first posting of a transaction only.
 --
-postingsReportItemAsText :: CliOpts -> PostingsReportItem -> String
-postingsReportItemAsText opts (mdate, menddate, mdesc, p, b) =
+-- Returns a string which can be multi-line, eg if the running balance
+-- has multiple commodities. Does not yet support formatting control
+-- like balance reports.
+--
+postingsReportItemAsText :: CliOpts -> Int -> Int -> PostingsReportItem -> String
+postingsReportItemAsText opts preferredamtwidth preferredbalwidth (mdate, menddate, mdesc, p, b) =
+  -- use elide*Width to be wide-char-aware
+  -- trace (show (totalwidth, datewidth, descwidth, acctwidth, amtwidth, balwidth)) $
   intercalate "\n" $
-    [printf ("%-"++datew++"s %-"++descw++"s  %-"++acctw++"s  %"++amtw++"s  %"++balw++"s")
-            date desc acct amtfirstline balfirstline]
+    [concat [fitString (Just datewidth) (Just datewidth) True True date
+            ," "
+            ,fitString (Just descwidth) (Just descwidth) True True desc
+            ,"  "
+            ,fitString (Just acctwidth) (Just acctwidth) True True acct
+            ,"  "
+            ,fitString (Just amtwidth) (Just amtwidth) True False amtfirstline
+            ,"  "
+            ,fitString (Just balwidth) (Just balwidth) True False balfirstline
+            ]]
     ++
-    [printf (spacer ++ "%"++amtw++"s  %"++balw++"s") a b | (a,b) <- zip amtrest balrest ]
-
+    [concat [spacer
+            ,fitString (Just amtwidth) (Just amtwidth) True False a
+            ,"  "
+            ,fitString (Just balwidth) (Just balwidth) True False b
+            ]
+     | (a,b) <- zip amtrest balrest
+     ]
     where
       -- calculate widths
       (totalwidth,mdescwidth) = registerWidthsFromOpts opts
-      amtwidth = 12
-      balwidth = 12
       (datewidth, date) = case (mdate,menddate) of
                             (Just _, Just _)   -> (21, showDateSpan (DateSpan mdate menddate))
                             (Nothing, Just _)  -> (21, "")
                             (Just d, Nothing)  -> (10, showDate d)
                             _                  -> (10, "")
+      (amtwidth, balwidth)
+        | shortfall <= 0 = (preferredamtwidth, preferredbalwidth)
+        | otherwise      = (adjustedamtwidth, adjustedbalwidth)
+        where
+          mincolwidth = 2 -- columns always show at least an ellipsis
+          maxamtswidth = max 0 (totalwidth - (datewidth + 1 + mincolwidth + 2 + mincolwidth + 2 + 2))
+          shortfall = (preferredamtwidth + preferredbalwidth) - maxamtswidth
+          amtwidthproportion = fromIntegral preferredamtwidth / fromIntegral (preferredamtwidth + preferredbalwidth)
+          adjustedamtwidth = round $ amtwidthproportion * fromIntegral maxamtswidth
+          adjustedbalwidth = maxamtswidth - adjustedamtwidth
+
       remaining = totalwidth - (datewidth + 1 + 2 + amtwidth + 2 + balwidth)
       (descwidth, acctwidth)
         | hasinterval = (0, remaining - 2)
@@ -137,16 +170,16 @@ postingsReportItemAsText opts (mdate, menddate, mdesc, p, b) =
         where
             hasinterval = isJust menddate
             w = fromMaybe ((remaining - 2) `div` 2) mdescwidth
-      [datew,descw,acctw,amtw,balw] = map show [datewidth,descwidth,acctwidth,amtwidth,balwidth]
 
       -- gather content
-      desc = maybe "" (take descwidth . elideRight descwidth) mdesc
+      desc = fromMaybe "" mdesc
       acct = parenthesise $ elideAccountName awidth $ paccount p
          where
-          (parenthesise, awidth) = case ptype p of
-                               BalancedVirtualPosting -> (\s -> "["++s++"]", acctwidth-2)
-                               VirtualPosting         -> (\s -> "("++s++")", acctwidth-2)
-                               _                      -> (id,acctwidth)
+          (parenthesise, awidth) =
+            case ptype p of
+              BalancedVirtualPosting -> (\s -> "["++s++"]", acctwidth-2)
+              VirtualPosting         -> (\s -> "("++s++")", acctwidth-2)
+              _                      -> (id,acctwidth)
       amt = showMixedAmountWithoutPrice $ pamount p
       bal = showMixedAmountWithoutPrice b
       -- alternate behaviour, show null amounts as 0 instead of blank
@@ -159,9 +192,6 @@ postingsReportItemAsText opts (mdate, menddate, mdesc, p, b) =
       (balfirstline:balrest) = take numlines $ replicate (numlines - ballen) "" ++ ballines -- balance amount is bottom-aligned
       spacer = replicate (totalwidth - (amtwidth + 2 + balwidth)) ' '
 
--- XXX
--- showPostingWithBalanceForVty showtxninfo p b = postingsReportItemAsText defreportopts $ mkpostingsReportItem showtxninfo p b
-
 tests_Hledger_Cli_Register :: Test
 tests_Hledger_Cli_Register = TestList
   tests_postingsReportAsText
